package space.eptaproject.vmodextras;

import it.unimi.dsi.fastutil.objects.ObjectArrayList;
import it.unimi.dsi.fastutil.objects.ObjectImmutableList;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.common.ForgeConfigSpec;
import net.minecraftforge.registries.ForgeRegistries;

import java.util.*;

public class Config {
    public static final ForgeConfigSpec COMMON_CONFIG;
    public static final ForgeConfigSpec.ConfigValue<List<? extends String>> RESOURCE_LOCATION_LIST;
    public static final ForgeConfigSpec.ConfigValue<List<? extends String>> ITEM_BLACK_LIST;
    public static final ForgeConfigSpec.ConfigValue<String> RESOURCE_LOCATION;

    public static final ForgeConfigSpec.ConfigValue<Integer> Suspension_Stiffness_LIMIT;
    public static final ForgeConfigSpec.ConfigValue<Double> Suspension_Stiffness_delta;

    public static final ForgeConfigSpec.ConfigValue<Boolean> Schematic_Blocked_Blocks;
    public static final ForgeConfigSpec.ConfigValue<Boolean> Inventory_Blocked_Blocks;

    public static List<ResourceLocation> parsedList = new ObjectArrayList<>();
    private static List<Map.Entry<Integer, ResourceLocation>> ITEM = new ObjectImmutableList<>(new ArrayList<>());
    public static ResourceLocation replace = Validator.RB;
    public static boolean Inventory_Blocked_Blocks_Enable = false;
    public static boolean Schematic_Blocked_Blocks_Enable = false;

    static {
        ForgeConfigSpec.Builder builder = new ForgeConfigSpec.Builder();

        builder.push("Schematic");

        RESOURCE_LOCATION_LIST = builder
                .comment("List of blocked blocks, format: 'modid:path'")
                .defineListAllowEmpty(
                        "blocked",
                        List.of("minecraft:stone", "minecraft:diamond_block"),
                        obj -> obj instanceof String && ResourceLocation.isValidResourceLocation((String) obj)
                );

        RESOURCE_LOCATION = builder
                .comment("replace to, format: 'modid:path'")
                .define(
                        "replace",
                        Validator.RB.toString()
                );

        Schematic_Blocked_Blocks = builder
                .define("enable", false);


        builder.pop();

        builder.push("Inventory");

        ITEM_BLACK_LIST = builder
                .comment("List of blocked items, format: 'modid:path'")
                .defineListAllowEmpty(
                        "blocked",
                        List.of("minecraft:stone", "minecraft:diamond_block"),
                        obj -> obj instanceof String && ResourceLocation.isValidResourceLocation((String) obj)
                );

        Inventory_Blocked_Blocks = builder
                .define("enable", false);

        builder.pop();

        builder.push("TrackWork");

        Suspension_Stiffness_LIMIT = builder
                .comment("track suspension stiffness")
                .defineInRange("SuspensionStiffness", 4, 1, 20);

        Suspension_Stiffness_delta = builder
                .comment("track suspension stiffness delta")
                .defineInRange("suspension_stiffness_delta", 1d, 0.10d, 1d);


        builder.pop();
        COMMON_CONFIG = builder.build();
    }

    public static void bake() {
        parsedList = RESOURCE_LOCATION_LIST.get().stream()
                .map(ResourceLocation::tryParse)
                .filter(Objects::nonNull)
                .toList();

        ITEM = new ObjectImmutableList(ITEM_BLACK_LIST.get().stream()
                .map(s -> {
                    try {
                        ResourceLocation location = ResourceLocation.tryParse(s);
                        return new AbstractMap.SimpleEntry<Integer, ResourceLocation>(location.hashCode(), location);
                    } catch (Exception e) {
                        return null;
                    }
                })
                .filter(Objects::nonNull)
                .toList());
        try {
            replace = new ResourceLocation(RESOURCE_LOCATION.get());
        } catch (Exception e) {
            replace = Validator.RB;
        }

        Inventory_Blocked_Blocks_Enable = Inventory_Blocked_Blocks.get();
        Schematic_Blocked_Blocks_Enable = Schematic_Blocked_Blocks.get();
    }

    public static boolean contains(ItemStack stack) {
        if (stack.isEmpty() || !Inventory_Blocked_Blocks_Enable) return false;
        ResourceLocation id = ForgeRegistries.ITEMS.getKey(stack.getItem());
        if (id == null) return false;

        int hash = id.hashCode();
        for (var entry : ITEM) {
            if (entry.getKey() == hash && entry.getValue().equals(id)) {
                return true;
            }
        }
        return false;
    }
}
