package space.eptaproject.vmodextras.compact.trackwork.block.entity;

import edn.stratodonut.drivebywire.wire.ShipWireNetworkManager;
import edn.stratodonut.trackwork.tracks.forces.PhysicsTrackController;
import edn.stratodonut.trackwork.tracks.forces.SimpleWheelController;
import net.minecraft.core.BlockPos;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.Tag;
import net.minecraft.world.level.block.Rotation;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockState;

import org.valkyrienskies.core.api.ships.ServerShip;
import org.valkyrienskies.mod.common.VSGameUtilsKt;
import space.eptaproject.vmodextras.compact.trackwork.TrackWorkCompact;
import space.eptaproject.vmodextras.compact.trackwork.IPhysicsTrackController;
import space.eptaproject.vmodextras.compact.trackwork.ISimpleWheelController;


public class CompactBackupBlockEntity extends BlockEntity {
    private CompoundTag pendingBackupData;

    public CompactBackupBlockEntity(BlockPos p_155229_, BlockState p_155230_) {
        super(TrackWorkCompact.COMPACT_BACKUP_BLOCK_ENTITY.get(), p_155229_, p_155230_);
    }

    @Override
    protected void saveAdditional(CompoundTag tag) {
        super.saveAdditional(tag);
        ServerShip ship = (ServerShip) VSGameUtilsKt.getShipObjectManagingPos(this.level, getBlockPos());
        if (ship != null) {
            {
                PhysicsTrackController physicsTrackController = ship.getAttachment(PhysicsTrackController.class);
                if (physicsTrackController != null) {
                    IPhysicsTrackController controller = (IPhysicsTrackController) physicsTrackController;
                    CompoundTag local = new CompoundTag();

                    controller.encode(local);

                    tag.put("PhysicsTrackController", local);
                }
            }

            {
                SimpleWheelController simpleWheelController = ship.getAttachment(SimpleWheelController.class);
                if (simpleWheelController != null) {
                    ISimpleWheelController controller = (ISimpleWheelController) simpleWheelController;
                    CompoundTag local = new CompoundTag();

                    controller.encode(local);

                    tag.put("SimpleWheelController", local);
                }
            }
        }

        ship = (ServerShip) VSGameUtilsKt.getShipManagingPos(this.level, this.getBlockPos());

        if (this.level != null) {
            if (this.pendingBackupData == null) {
                this.pendingBackupData = new CompoundTag();
            }

            ShipWireNetworkManager.get(ship).ifPresent((m) -> this.pendingBackupData.merge(m.serialiseToNbt(this.level, this.getBlockPos())));

            if (this.pendingBackupData != null) {
                tag.put("WireNetwork", this.pendingBackupData);
            }

        }
    }

    @Override
    public void load(CompoundTag tag) {
        super.load(tag);

        ServerShip ship = (ServerShip) VSGameUtilsKt.getShipObjectManagingPos(this.level, getBlockPos());
        if (ship != null) {
            if (tag.contains("SimpleWheelController", Tag.TAG_COMPOUND)) {
                ((ISimpleWheelController) SimpleWheelController.getOrCreate(ship)).decode(tag.getCompound("SimpleWheelController"));
            }

            if (tag.contains("PhysicsTrackController", Tag.TAG_COMPOUND)) {
                ((IPhysicsTrackController) PhysicsTrackController.getOrCreate(ship)).decode(tag.getCompound("PhysicsTrackController"));
            }
        }

        ship = (ServerShip) VSGameUtilsKt.getShipManagingPos(this.level, getBlockPos());
        if (ship != null) {
            if (tag.contains("WireNetwork", 10)) {
                ShipWireNetworkManager.loadIfNotExists(ship, level, tag.getCompound("WireNetwork"), this.getBlockPos(), Rotation.NONE);

                this.pendingBackupData = tag.getCompound("WireNetwork");
            }
        }
    }
}
